package controleurs;

import java.io.*;

import com.fasterxml.jackson.databind.ObjectMapper;


import dao.IngredientDAODatabase;
import dto.Ingredient;
import jakarta.servlet.ServletException;
import jakarta.servlet.annotation.WebServlet;
import jakarta.servlet.http.HttpServlet;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;


/*Créez une servlet contrôleur IngredientRestAPI.java. Dotez cette classe d’un endpoint /ingrédients qui
renvoie la collection d’ingrédients en JSON. */

@WebServlet("/ingredients/*")
public class IngredientRestAPI extends HttpServlet{

    private IngredientDAODatabase ingredientDAOList = new IngredientDAODatabase();

    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json; charset=UTF-8");
        String pathInfo = request.getPathInfo();
        PrintWriter out = response.getWriter();
        
        if (pathInfo != null) {
            String[] pathParts = pathInfo.split("/");
            // si /ingrédients/{id}/name alors on renvoie le nom de l'ingrédient {id}
            if (pathParts.length > 2 && pathParts[2].equals("name")) {
                int id = Integer.parseInt(pathParts[1]);
                Ingredient ingredient = ingredientDAOList.findById(id);
                if (ingredient == null) {
                    response.sendError(HttpServletResponse.SC_NOT_FOUND);
                } else {
                    out.print(ingredient.getName());
                    response.setStatus(HttpServletResponse.SC_FOUND);
                }
            }
            // si /ingrédients/{id} alors on renvoie l'ingrédient {id} en JSON
            else if (pathParts.length > 1) {
                int id = Integer.parseInt(pathParts[1]);
                Ingredient ingredient = ingredientDAOList.findById(id);
                if (ingredient == null) {
                    response.sendError(HttpServletResponse.SC_NOT_FOUND);
                } else {
                    ObjectMapper mapper = new ObjectMapper();
                    response.setStatus(HttpServletResponse.SC_FOUND);
                    out.print(mapper.writeValueAsString(ingredient));

                }
            }
        }
        // si /ingrédients alors on renvoie la liste des ingrédients en JSON 
        else {
            ObjectMapper mapper = new ObjectMapper();
            response.setStatus(HttpServletResponse.SC_OK);
            out.println(mapper.writeValueAsString(ingredientDAOList.findAll()));
        }
    }

    public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json; charset=UTF-8");
        PrintWriter out = response.getWriter();
        //récupère le corps de la requête
        BufferedReader reader = request.getReader();
        String body = reader.readLine();
        ObjectMapper mapper = new ObjectMapper();
        Ingredient ingredient = mapper.readValue(body, Ingredient.class);
        // si l'ingrédient existe déjà
        if (ingredientDAOList.findById(ingredient.getId()) != null) {
            //renvoie une erreur 409
            response.sendError(HttpServletResponse.SC_CONFLICT);
        } else {
            //sinon on l'ajoute
            ingredientDAOList.save(ingredient);
            //et renvoi un message  is created
            response.setStatus(HttpServletResponse.SC_CREATED);
            out.println(mapper.writeValueAsString(ingredient));
        }
    }

    public void doDelete(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json; charset=UTF-8");
        
        PrintWriter out = response.getWriter();
        // si /ingrédients/1 alors on supprime l'ingrédient 1
        String pathInfo = request.getPathInfo();
        if (pathInfo != null) {
            String[] pathParts = pathInfo.split("/");
            if (pathParts.length > 1) {
                int id = Integer.parseInt(pathParts[1]);
                // regarde si l'ingrédient existe
                if (ingredientDAOList.findById(id) == null) {
                    response.sendError(HttpServletResponse.SC_NOT_FOUND);


                } else {
                    //sinon on le supprime
                    ingredientDAOList.delete(id);
                    //et renvoi un message  is le json de l'ingrédient supprimé
                    response.setStatus(HttpServletResponse.SC_OK);
                                                          
                }
            }
        }
    }
    
    public void doPut(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json; charset=UTF-8");
        PrintWriter out = response.getWriter();
        //récupère le corps de la requête
        BufferedReader reader = request.getReader();
        String body = reader.readLine();
        ObjectMapper mapper = new ObjectMapper();
        Ingredient ingredient = mapper.readValue(body, Ingredient.class);
        // si l'ingrédient n'existe pas
        if (ingredientDAOList.findById(ingredient.getId()) == null) {
            //renvoie une erreur 404
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
        } else {
            //sinon on le met à jour
            ingredientDAOList.update(ingredient);

            response.setStatus(HttpServletResponse.SC_OK);
            out.println(mapper.writeValueAsString(ingredient));
          
        }
    }


    public void doPatch(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json; charset=UTF-8");
        //récupère le corps de la requête
        BufferedReader reader = request.getReader();
        String body = reader.readLine();
        ObjectMapper mapper = new ObjectMapper();
        Ingredient ingredient = mapper.readValue(body, Ingredient.class);
        // récupère l'id de l'ingrédient à modifier
        String pathInfo = request.getPathInfo();
        String[] pathParts = pathInfo.split("/");
        int id = Integer.parseInt(pathParts[1]);
        // si l'ingrédient n'existe pas
        if (ingredientDAOList.findById(id) == null){
            //renvoie une erreur 404
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
        } else {
            //sinon on le met à jour
            ingredientDAOList.patch(id,ingredient);

            response.setStatus(HttpServletResponse.SC_OK);
            doGet(request, response);
        }
    }



    // faire une méthode service qui redirige vers les bonnes méthodes en fonction de la méthode de la requête
    public void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        if (request.getMethod().equals("GET")) {
            doGet(request, response);
        } else if (request.getMethod().equals("POST")) {
            doPost(request, response);
        } else if (request.getMethod().equals("DELETE")) {
            doDelete(request, response);
        } else if (request.getMethod().equals("PUT")) {
            doPut(request, response);
        } else if (request.getMethod().equals("PATCH")) {
            doPatch(request, response);
        }
    }
}


