package controleurs;

import jakarta.servlet.ServletException;
import jakarta.servlet.annotation.WebServlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

import com.fasterxml.jackson.databind.ObjectMapper;

import dao.PizzasDAO;
import dto.Pizza;
import dto.PizzaCompo;
import jakarta.servlet.http.HttpServlet;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

@WebServlet("/pizzas/*")
public class PizzasRestAPI extends HttpServlet{
    private PizzasDAO pizzasDAO = new PizzasDAO();

    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
        response.setContentType("application/json; charset=UTF-8");
        String pathInfo = request.getPathInfo();
        PrintWriter out = response.getWriter();
        ObjectMapper mapper = new ObjectMapper();
        if (pathInfo != null) {
            String[] pathParts = pathInfo.split("/");
            if (pathParts.length > 2 && pathParts[2].equals("prixfinal")) {
                String nom = pathParts[1];
                if (pizzasDAO.findByName(nom) != null){
                    PizzaCompo pizza = pizzasDAO.findByName(nom);
                    out.print(pizzasDAO.totalPrice(nom));
                    response.setStatus(HttpServletResponse.SC_FOUND);
                    
                } else {
                    response.sendError(HttpServletResponse.SC_NOT_FOUND);
                    out.println("La pizza n'existe pas");
                }
            }
            else if (pathParts.length > 1) {
                String nom = pathInfo.substring(1);
                if (pizzasDAO.findByName(nom) != null){
                    Pizza pizza = pizzasDAO.findByName(nom);
                    out.println(mapper.writeValueAsString(pizza));
                    response.setStatus(HttpServletResponse.SC_FOUND);
                } else {
                    response.sendError(HttpServletResponse.SC_NOT_FOUND);
                }
            }
        } else {
            List<Pizza> pizzas = pizzasDAO.findAll();
            response.setStatus(HttpServletResponse.SC_OK);
            out.println(mapper.writeValueAsString(pizzas));
        }            
    }

    // Ajoutez une nouvelle pizza et ses ingrédients à la base de données 
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {
        response.setContentType("application/json; charset=UTF-8");
        PrintWriter out = response.getWriter();
        ObjectMapper mapper = new ObjectMapper();
        PizzaCompo pizza = mapper.readValue(request.getReader(), PizzaCompo.class);
        if (pizzasDAO.findByName(pizza.getNom()) != null){
            response.sendError(HttpServletResponse.SC_CONFLICT);
            
            
        }else {
            pizzasDAO.save(pizza);
            out.println(mapper.writeValueAsString(pizza));
            response.setStatus(HttpServletResponse.SC_CREATED);
        }
        

    }
    // Supprimez une pizza de la base de données
    public void doDelete(HttpServletRequest request, HttpServletResponse response) throws IOException {
        response.setContentType("application/json; charset=UTF-8");
        String pathInfo = request.getPathInfo();
        PrintWriter out = response.getWriter();
       // si /pizza/chorizo/1 alors on supprime la relation entre la pizza et l'ingrédient 1 de la table pizzas_ingredient
        if (pathInfo != null) {
            String[] pathParts = pathInfo.split("/");
            if (pathParts.length > 2) {
                String nom = pathParts[1];
                int id = Integer.parseInt(pathParts[2]);
                if (pizzasDAO.findByName(nom) != null){
                    // regarde si l'ingrédient existe dans la table pizzas_ingredient
                    if (!pizzasDAO.findPizzaIngredient(nom, id)) {
                        response.sendError(HttpServletResponse.SC_NOT_FOUND);
                    }
                    else{                    
                    pizzasDAO.deletePizzaIngredient(nom, id);
                    response.setStatus(HttpServletResponse.SC_OK);
                    }
                } else {
                    response.sendError(HttpServletResponse.SC_NOT_FOUND);
                }
            }
            // si /pizza/chorizo alors on supprime la pizza de la table pizzas
            else {
                String nom = pathInfo.substring(1);
                if (pizzasDAO.findByName(nom) != null){
                    pizzasDAO.delete(nom);
                    response.setStatus(HttpServletResponse.SC_OK);
                } else {
                    response.sendError(HttpServletResponse.SC_NOT_FOUND);
                }
            }
        }
       
       
    }

    @Override
    public void doPut(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json; charset=UTF-8");
        PrintWriter out = response.getWriter();
        ObjectMapper mapper = new ObjectMapper();
        PizzaCompo pizza = mapper.readValue(request.getReader(), PizzaCompo.class);
        if (pizzasDAO.findByName(pizza.getNom()) != null){
            pizzasDAO.update(pizza);
            response.setStatus(HttpServletResponse.SC_OK);
            doGet(request, response);
        } else {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
        }
    }


    public void doPatch(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json; charset=UTF-8");
        PrintWriter out = response.getWriter();
        ObjectMapper mapper = new ObjectMapper();
        PizzaCompo pizza = mapper.readValue(request.getReader(), PizzaCompo.class);

        String pathInfo = request.getPathInfo();
        String[] pathParts = pathInfo.split("/");
        String nom = pathParts[1];
        
        if( pizzasDAO.findByName(nom) != null){
            pizzasDAO.patchPizza(nom, pizza);
            response.setStatus(HttpServletResponse.SC_OK);
            doGet(request, response);
        } else {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
        }
    }


    @Override
    protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        if(request.getMethod().equals("PUT")) {
            doPut(request, response);
        } else if (request.getMethod().equals("DELETE")) {
            doDelete(request, response);
        } else if (request.getMethod().equals("POST")) {
            doPost(request, response);
        } else if(request.getMethod().equals("GET")) {
            doGet(request, response);     
        } else if (request.getMethod().equals("PATCH")) {
            doPatch(request, response);
        }
    }
}
