package controleurs;

import java.io.PrintWriter;

import com.fasterxml.jackson.databind.ObjectMapper;

import dao.CommandeDAO;
import dto.Commande;
import dto.CommandeDetail;
import jakarta.servlet.ServletException;
import jakarta.servlet.annotation.WebServlet;
import jakarta.servlet.http.*;
import java.io.IOException;


@WebServlet("/commandes/*")
public class CommandeRestAPI extends HttpServlet{

    private CommandeDAO commandeDAO = new CommandeDAO();
    
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json; charset=UTF-8");
        String pathInfo = request.getPathInfo();
        PrintWriter out = response.getWriter();
        
        if (pathInfo != null) {
            String[] pathParts = pathInfo.split("/");
            // si /commandes/{id}/prixtotal alors on renvoie le prix total de la commande {id}
            if (pathParts.length > 2 && pathParts[2].equals("prixfinal")) {
                int id = Integer.parseInt(pathParts[1]);
    
                if (commandeDAO.findById(id) == null) {
                    response.sendError(HttpServletResponse.SC_NOT_FOUND);
                } else {
                    out.print(commandeDAO.totalPrice(id));
                    response.setStatus(HttpServletResponse.SC_FOUND);
                }
            }
            // si /commandes/{id} alors on renvoie la commande {id} en JSON
            else if (pathParts.length > 1) {
                int id = Integer.parseInt(pathParts[1]);
                if (commandeDAO.findById(id) == null) {
                    response.sendError(HttpServletResponse.SC_NOT_FOUND);
                } else {
                    ObjectMapper mapper = new ObjectMapper();
                    out.print(mapper.writeValueAsString(commandeDAO.findById(id)));
                    response.setStatus(HttpServletResponse.SC_FOUND);
                }
            }
        }
        // si /commandes alors on renvoie toutes les commandes en JSON
        else {
            ObjectMapper mapper = new ObjectMapper();
            out.println(mapper.writeValueAsString(commandeDAO.findAll()));
            response.setStatus(HttpServletResponse.SC_OK);
        }
    }

    public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json; charset=UTF-8");
        PrintWriter out = response.getWriter();
        ObjectMapper mapper = new ObjectMapper();
        CommandeDetail commande = mapper.readValue(request.getReader(), CommandeDetail.class);
        // si la commande existe déjà, on renvoie une erreur  conflict
        if (commandeDAO.findById(commande.getId()) != null) {
            response.sendError(HttpServletResponse.SC_CONFLICT);
            
        }else{
            // sinon on l'ajoute à la base de données
            commandeDAO.save(commande);
            response.setStatus(HttpServletResponse.SC_CREATED);
            out.println(mapper.writeValueAsString(commande));
        }
    }

    public void doDelete(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json; charset=UTF-8");
        String pathInfo = request.getPathInfo();
        PrintWriter out = response.getWriter();
        
        if (pathInfo != null) {
            String[] pathParts = pathInfo.split("/");
            // si /commandes/{id}/{pizza} alors on supprime la pizza {pizza} de la commande {id}
            if (pathParts.length > 2) {
                int id = Integer.parseInt(pathParts[1]);
                // si la commande {id} n'existe pas, on renvoie une erreur 404
                if (commandeDAO.findById(id) == null) {
                    response.sendError(HttpServletResponse.SC_NOT_FOUND);
                }else {
                    String pizza = pathParts[2];
                    // si la pizza {pizza} n'existe pas dans la commande {id}, on renvoie une erreur 404
                    if (commandeDAO.commandeFindPizza(id, pizza)) {
                        commandeDAO.deleteCommandeDetail(id, pizza);
                        response.setStatus(HttpServletResponse.SC_OK);
                    } else {
                        response.sendError(HttpServletResponse.SC_NOT_FOUND);
                    }
                }
            }
            // sinon on supprime la commande {id}
            else if (pathParts.length > 1) {
                
                int id = Integer.parseInt(pathParts[1]);
                if (commandeDAO.findById(id) == null) {
                    response.sendError(HttpServletResponse.SC_NOT_FOUND);
                } else {
                commandeDAO.deleteCommande(id);
                response.setStatus(HttpServletResponse.SC_OK);
                }
            }
        }        
    }
    
    public void doPut(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json; charset=UTF-8");
        PrintWriter out = response.getWriter();
        ObjectMapper mapper = new ObjectMapper();
        CommandeDetail commande = mapper.readValue(request.getReader(), CommandeDetail.class);
        if (commandeDAO.findById(commande.getId()) == null) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
        } else {
            commandeDAO.update(commande);
            doGet(request, response);
        }

    }

    public void doPatch(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        
        response.setContentType("application/json; charset=UTF-8");
        PrintWriter out = response.getWriter();
        ObjectMapper mapper = new ObjectMapper();
        CommandeDetail commande = mapper.readValue(request.getReader(), CommandeDetail.class);

        String pathInfo = request.getPathInfo();
        String[] pathParts = pathInfo.split("/");
        int id = Integer.parseInt(pathParts[1]);
        if (commandeDAO.findById(id) == null) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
        } else {
            commande.setId(id);
            commandeDAO.commandePatch(id, commande);
            
            response.setStatus(HttpServletResponse.SC_OK);
            doGet(request, response);
        }
    }
    @Override
    public void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        if (request.getMethod().equals("GET")) {
            doGet(request, response);
        } else if (request.getMethod().equals("POST")) {
            doPost(request, response);
        } else if (request.getMethod().equals("DELETE")) {
            doDelete(request, response);
        } else if (request.getMethod().equals("PUT")) {
            doPut(request, response);
        } else if (request.getMethod().equals("PATCH")) {
            doPatch(request, response);
        }
    }
    
}
